package org.dive4elements.river.importer.parsers;

import java.math.BigDecimal;

import java.text.ParseException;

import org.apache.log4j.Logger;

import org.dive4elements.river.importer.ImportBedHeight;
import org.dive4elements.river.importer.ImportBedHeightSingle;
import org.dive4elements.river.importer.ImportBedHeightSingleValue;


public class BedHeightSingleParser extends BedHeightParser {

    private static final Logger log =
        Logger.getLogger(BedHeightSingleParser.class);



    @Override
    protected ImportBedHeight newImportBedHeight(String description) {
        return new ImportBedHeightSingle(description);
    }



    /**
     * Create ImportBedHeightSingleValue from a line of csv file, add
     * it to the ImportBedHeight.
     */
    @Override
    protected void handleDataLine(ImportBedHeight obj, String line) {
        String[] values = line.split(SEPERATOR_CHAR);

        if (values == null || (values.length != 1 && values.length < 6)) {
            //log.warn("BSP: Error while parsing data line: '" + line + "'");
            return;
        }

        BigDecimal km;

        try {
            km = new BigDecimal(nf.parse(values[0]).doubleValue());
        }
        catch (ParseException e) {
            // We expect a lot of ";;;;;;" lines.
            //log.warn("BSP: Error while parsing km of data row.", e);
            return;
        }

        // Handle gaps like "10,0;;;;;".
        if (values.length == 1) {
            ImportBedHeightSingleValue value = new ImportBedHeightSingleValue(
                (ImportBedHeightSingle) obj,
                km,
                null, null, null, null, null);
            obj.addValue(value);
            return;
        }

        // Because we cannot enforce consistency of values with complete data
        // via null constraints in the database (as there are "gap" values),
        // do this checks manually.
        if (values[3].length() == 0 || values[4].length() == 0
            || values[5].length() == 0) {
            //log.warn("BSP: Error while parsing data row (manual null constraint violated).");
            return;
        }

        try {
            ImportBedHeightSingleValue value = new ImportBedHeightSingleValue(
                (ImportBedHeightSingle) obj,
                km,
                new BigDecimal(nf.parse(values[1]).doubleValue()),
                new BigDecimal(nf.parse(values[2]).doubleValue()),
                new BigDecimal(nf.parse(values[3]).doubleValue()),
                parseBigDecimal(values[4], line),
                new BigDecimal(nf.parse(values[5]).doubleValue())
            );

            obj.addValue(value);
        }
        catch (ParseException e) {
            log.warn("BSP: unparseable value in data row.", e);
        }
    }
    
    private BigDecimal parseBigDecimal(String value, String line) {
        BigDecimal result = null;
        try {
            Double dValue = Double.valueOf(value.replace(",", "."));
            result = new BigDecimal(dValue.doubleValue());
        }
        catch (NumberFormatException nfe) {
            log.warn(
                "Could not parse " +
                value +
                " in bed heigt single row: "
                + line);
        }
        return result;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
